@@ -1,17 +1,19 @@
 # Load libs
 import json
-import os
+import logging
 import math
+import os
+import shutil
 import sys
+
 import numpy as np
 import pandas as pd
-import logging
-from ts_rnn.logger import logger
-from ts_rnn.config import DEFAULT_HP, DEFAULT_ARCH
-from tensorflow.keras.models import Sequential, load_model
-from ts_rnn.utils import split_sequence, history_plot, timeit
 from keras_tuner import RandomSearch, BayesianOptimization, Hyperband
-from tensorflow.keras.layers import Dense, Dropout, LSTM, GRU, Bidirectional, BatchNormalization, SimpleRNN, RNN
+from tensorflow.keras.layers import Dense, Dropout, LSTM, GRU, Bidirectional, BatchNormalization, SimpleRNN
+from tensorflow.keras.models import Sequential, load_model
+
+from ts_rnn import config, utils
+from ts_rnn.logger import logger
 
 
 class TS_RNN:
@@ -63,8 +65,8 @@ class TS_RNN:
         # Set model arch
         if rnn_arch is None:
             self.logger.warning(f'rnn_arch is not defined. Model will be compiled with default architecture')
-            self.rnn_arch = DEFAULT_ARCH
-            self.hp = DEFAULT_HP
+            self.rnn_arch = config.DEFAULT_ARCH
+            self.hp = config.DEFAULT_HP
         elif (rnn_arch is not None) and (tuner_hp is None):
             self.logger.warning(f'tuner_hp is not defined. Model will be trained without tuning architecture')
             self.rnn_arch = rnn_arch
@@ -99,10 +101,10 @@ class TS_RNN:
             self.logger.exception('Define n_lags and horizon')
             raise AssertionError('Define n_lags and horizon')
         try:
-            assert self.strategy in ["Direct", "Recursive", "MiMo", "DirRec", "DirMo"]
+            assert self.strategy in config.MODEL_STRATEGIES
         except AssertionError:
-            self.logger.exception('Use strategy from ["Direct", "Recursive", "MiMo", "DirRec", "DirMo"]')
-            raise AssertionError('Use strategy from ["Direct", "Recursive", "MiMo", "DirRec", "DirMo"]')
+            self.logger.exception(f'Use strategy from {config.MODEL_STRATEGIES}')
+            raise AssertionError(f'Use strategy from {config.MODEL_STRATEGIES}')
 
         try:
             assert self.tuner in ["RandomSearch", "BayesianOptimization", "Hyperband"]
@@ -230,7 +232,7 @@ class TS_RNN:
 
         return _model
 
-    @timeit
+    @utils.timeit
     def fit(self,
             target_train,
             target_val=None,
@@ -302,9 +304,9 @@ class TS_RNN:
                                                                  )
 
                 if self.save_dir is not None and ((validation_split != 0) or (validation_data is not None)):
-                    history_plot(history,
-                                 self.save_dir,
-                                 show=True if verbose > 0 else False)
+                    utils.history_plot(history,
+                                       self.save_dir,
+                                       show=True if verbose > 0 else False)
             else:
                 if self.strategy == "DirRec":
                     self.n_lags = true_n_lags + model_id  # needed to train DirRec strategy
@@ -321,6 +323,9 @@ class TS_RNN:
                 self.model_list[model_id]["model"] = self.model_list[model_id]["tuner"].get_best_models(num_models=1)[0]
                 if self.strategy == "DirRec":
                     self.n_lags = true_n_lags
+
+        if self.save_dir is not None:
+            shutil.rmtree(os.path.join(self.save_dir, 'TS_RNN_tuner_log'), ignore_errors=True)
         self.logger.info('[Training] Training ended')
         return self
 
@@ -361,7 +366,7 @@ class TS_RNN:
 
         return target_train, target_val, factors_train, factors_val
 
-    @timeit
+    @utils.timeit
     def predict(self, target, prediction_len, factors=None):
         """
         Prediction with auto-set method based by rnn_arch
@@ -405,8 +410,8 @@ class TS_RNN:
         elif self.strategy == 'DirMo':
             predicted = self._dirmo_pred(input_df)
         else:
-            self.logger.critical(f'Use strategy from ["Direct", "Recursive", "MiMo", "DirRec", "DirMo"]')
-            raise AssertionError(f'Use strategy from ["Direct", "Recursive", "MiMo", "DirRec", "DirMo"]')
+            self.logger.critical(f'Use strategy from {config.MODEL_STRATEGIES}')
+            raise AssertionError(f'Use strategy from {config.MODEL_STRATEGIES}')
 
         self.logger.info(f'[Prediction] End predict by {self.strategy} strategy')
         return predicted.flatten()
@@ -557,19 +562,20 @@ class TS_RNN:
             raise AssertionError(
                 f'Define n_features in init of TS_RNN, factors given: {input_df.shape[1]}, n_features: {self.n_features}')
 
+        self._last_known_target = target_train.iloc[-self.n_lags:, :]
         if factors_train is not None:
-            self._last_known_factors = factors_train.iloc[-self.n_lags:, :-1]
+            self._last_known_factors = factors_train.iloc[-self.n_lags:, :]
 
         # split into samples
-        _X_train, _y_train = split_sequence(input_df.values,
-                                            n_steps_in=self.n_lags + _i_model if (
-                                                    self.strategy == "DirRec") else self.n_lags,
-                                            n_steps_out=self.n_step_out,
-                                            _full_out=True if ((factors is not None) and (
-                                                    self.strategy in ["Recursive", "DirRec"])) else False,
-                                            _i_model=_i_model if (self.strategy in ["Direct", 'DirMo']) else 0,
-                                            _start_ind=_i_model * self.n_step_out - _i_model if (
-                                                    self.strategy == "DirMo") else 0)
+        _X_train, _y_train = utils.split_sequence(input_df.values,
+                                                  n_steps_in=self.n_lags + _i_model if (
+                                                          self.strategy == "DirRec") else self.n_lags,
+                                                  n_steps_out=self.n_step_out,
+                                                  _full_out=True if ((factors is not None) and (
+                                                          self.strategy in ["Recursive", "DirRec"])) else False,
+                                                  _i_model=_i_model if (self.strategy in ["Direct", 'DirMo']) else 0,
+                                                  _start_ind=_i_model * self.n_step_out - _i_model if (
+                                                          self.strategy == "DirMo") else 0)
         if val_len == 0:
             _X_test, _y_test = None, None
         else:
@@ -587,8 +593,8 @@ class TS_RNN:
 
     def forecast(self, prediction_len):
         predicted = self.predict(
-            factors=np.array(self._last_known_factors) if self._last_known_factors is not None else None,
-            target=np.array(self._last_known_target),
+            factors=self._last_known_factors if self._last_known_factors is not None else None,
+            target=self._last_known_target,
             prediction_len=prediction_len)
         return predicted
 
@@ -600,14 +606,16 @@ class TS_RNN:
         for model_id in range(len(self.model_list)):
             self.model_list[model_id]["model"].summary()
 
-    def save(self, save_dir):
+    def save(self, model_name='tsrnn_model', save_dir='.'):
         """
         A method for saving model to h5 file
+        :param model_name: (str) name for your model
         :param save_dir: (str) path to h5 file with model
         :return: None
         """
-        models_folder_path = os.path.join(save_dir, "models")
-        os.makedirs(models_folder_path)
+        models_folder_path = os.path.join(save_dir, model_name)
+        if not os.path.exists(models_folder_path):
+            os.makedirs(models_folder_path)
         self.logger.info('[Model Saving] Saving model to file %s' % models_folder_path)
         with open(os.path.join(models_folder_path, 'ts_rnn.json'), 'w') as fp:
             json.dump({key: value for key, value in self.__dict__.items() if
@@ -625,10 +633,10 @@ def load_ts_rnn(path):
         params.update({'model_list': []})
     ts_rnn = TS_RNN(load=True)
     ts_rnn.__dict__ = params
+    ts_rnn.logger = logger
 
     for i, model_name in enumerate(sorted(os.listdir(path))):
         if '.h5' in model_name:
-            print(model_name)
             _model = load_model(os.path.join(path, model_name))
             ts_rnn.model_list.append({"model_name": model_name, "model": _model, 'tuner': None})
     return ts_rnn
